@@ -514,11 +514,12 @@ def get_hops(source_rse_id, dest_rse_id, include_multihop=False, multihop_rses=N
     if distance_graph.get(source_rse_id, {dest_rse_id: None}).get(dest_rse_id) is not None:
         # Check if there is a protocol match between the two RSEs
         try:
-            matching_scheme = rsemgr.find_matching_scheme(rse_settings_dest=limit_dest_schemes if limit_dest_schemes else __load_rse_settings(rse_id=dest_rse_id, session=session),
+            matching_scheme = rsemgr.find_matching_scheme(rse_settings_dest=__load_rse_settings(rse_id=dest_rse_id, session=session),
                                                           rse_settings_src=__load_rse_settings(rse_id=source_rse_id, session=session),
                                                           operation_src='third_party_copy',
                                                           operation_dest='third_party_copy',
-                                                          domain='wan')
+                                                          domain='wan',
+                                                          scheme=limit_dest_schemes if limit_dest_schemes else None)
             path = [{'source_rse_id': source_rse_id,
                      'dest_rse_id': dest_rse_id,
                      'source_scheme': matching_scheme[1],
@@ -573,11 +574,12 @@ def get_hops(source_rse_id, dest_rse_id, include_multihop=False, multihop_rses=N
                         continue
                     # Check if there is a compatible protocol pair
                     try:
-                        matching_scheme = rsemgr.find_matching_scheme(rse_settings_dest=limit_dest_schemes if out_v == dest_rse_id and limit_dest_schemes else __load_rse_settings(rse_id=out_v, session=session),
+                        matching_scheme = rsemgr.find_matching_scheme(rse_settings_dest=__load_rse_settings(rse_id=out_v, session=session),
                                                                       rse_settings_src=__load_rse_settings(rse_id=current_node, session=session),
                                                                       operation_src='third_party_copy',
                                                                       operation_dest='third_party_copy',
-                                                                      domain='wan')
+                                                                      domain='wan',
+                                                                      scheme=limit_dest_schemes if out_v == dest_rse_id and limit_dest_schemes else None)
                         visited_nodes[out_v] = {'distance': current_distance + distance_graph[current_node][out_v] + HOP_PENALTY,
                                                 'path': current_path + [{'source_rse_id': current_node,
                                                                          'dest_rse_id': out_v,
@@ -658,7 +660,6 @@ def get_transfer_requests_and_source_replicas(total_workers=0, worker_number=0,
     multi_hop_dict = {}
 
     rse_mapping = {}
-    current_schemes = SUPPORTED_PROTOCOLS
 
     multihop_rses = []
     try:
@@ -836,6 +837,11 @@ def get_transfer_requests_and_source_replicas(total_workers=0, worker_number=0,
                     dest_url = re.sub('https', 'gclouds', dest_url)
                     if source_protocol in ['davs', 'https']:
                         source_url += '?copy_mode=push'
+                elif sign_url == 's3':
+                    dest_url = re.sub('davs', 's3s', dest_url)
+                    dest_url = re.sub('https', 's3s', dest_url)
+                    if source_protocol in ['davs', 'https']:
+                        source_url += '?copy_mode=push'
                 elif WEBDAV_TRANSFER_MODE:
                     if source_protocol in ['davs', 'https']:
                         source_url += '?copy_mode=%s' % WEBDAV_TRANSFER_MODE
@@ -844,6 +850,11 @@ def get_transfer_requests_and_source_replicas(total_workers=0, worker_number=0,
                 if source_sign_url == 'gcs':
                     source_url = re.sub('davs', 'gclouds', source_url)
                     source_url = re.sub('https', 'gclouds', source_url)
+                elif source_sign_url == 's3':
+                    source_url = re.sub('davs', 's3s', source_url)
+                    source_url = re.sub('https', 's3s', source_url)
+
+                use_ipv4 = rse_attrs[source_rse_id].get('use_ipv4', False) or rse_attrs[dest_rse_id].get('use_ipv4', False)
 
                 # IV - get external_host + strict_copy + archive timeout
                 strict_copy = rse_attrs[dest_rse_id].get('strict_copy', False)
@@ -915,7 +926,7 @@ def get_transfer_requests_and_source_replicas(total_workers=0, worker_number=0,
                     file_metadata['previous_attempt_id'] = previous_attempt_id
 
                 transfers[req_id] = {'request_id': req_id,
-                                     'schemes': __add_compatible_schemes(schemes=[destination_protocol], allowed_schemes=current_schemes),
+                                     'schemes': __add_compatible_schemes(schemes=[destination_protocol], allowed_schemes=SUPPORTED_PROTOCOLS),
                                      'account': account,
                                      # 'src_urls': [source_url],
                                      'sources': [(rse, source_url, source_rse_id, ranking if ranking is not None else 0, link_ranking)],
@@ -935,8 +946,9 @@ def get_transfer_requests_and_source_replicas(total_workers=0, worker_number=0,
                     transfers[req_id]['initial_request_id'] = req_id
                 if strict_copy:
                     transfers[req_id]['strict_copy'] = strict_copy
-                if archive_timeout and (rses_info[dest_rse_id]['rse_type'] == RSEType.TAPE
-                                        or rses_info[dest_rse_id]['rse_type'] == 'TAPE'):
+                if use_ipv4:
+                    transfers[req_id]['use_ipv4'] = True
+                if archive_timeout and (rses_info[dest_rse_id]['rse_type'] == RSEType.TAPE or rses_info[dest_rse_id]['rse_type'] == 'TAPE'):
                     try:
                         transfers[req_id]['archive_timeout'] = int(archive_timeout)
                         logging.debug('Added archive timeout to transfer.')
@@ -944,7 +956,6 @@ def get_transfer_requests_and_source_replicas(total_workers=0, worker_number=0,
                         logging.warning('Could not set archive_timeout for %s. Must be integer.', dest_url)
                         pass
             else:
-                current_schemes = transfers[req_id]['schemes']
                 # parse allow tape source expression, not finally version.
                 allow_tape_source = dict_attributes.get('allow_tape_source', None)
 
@@ -964,6 +975,11 @@ def get_transfer_requests_and_source_replicas(total_workers=0, worker_number=0,
                     dest_url = re.sub('https', 'gclouds', dest_url)
                     if source_protocol in ['davs', 'https']:
                         source_url += '?copy_mode=push'
+                elif sign_url == 's3':
+                    dest_url = re.sub('davs', 's3s', dest_url)
+                    dest_url = re.sub('https', 's3s', dest_url)
+                    if source_protocol in ['davs', 'https']:
+                        source_url += '?copy_mode=push'
                 elif WEBDAV_TRANSFER_MODE:
                     if source_protocol in ['davs', 'https']:
                         source_url += '?copy_mode=%s' % WEBDAV_TRANSFER_MODE
@@ -972,6 +988,9 @@ def get_transfer_requests_and_source_replicas(total_workers=0, worker_number=0,
                 if source_sign_url == 'gcs':
                     source_url = re.sub('davs', 'gclouds', source_url)
                     source_url = re.sub('https', 'gclouds', source_url)
+                elif source_sign_url == 's3':
+                    source_url = re.sub('davs', 's3s', source_url)
+                    source_url = re.sub('https', 's3s', source_url)
 
                 # III - The transfer queued previously is a multihop, but this one is direct.
                 # Reset the sources, remove the multihop flag
@@ -1047,6 +1066,9 @@ def get_transfer_requests_and_source_replicas(total_workers=0, worker_number=0,
 
                 # transfers[id]['src_urls'].append((source_rse_id, source_url))
                 transfers[req_id]['sources'].append((rse, source_url, source_rse_id, ranking, link_ranking))
+                # if one source has force IPv4, force IPv4 for the whole job
+                if rse_attrs[source_rse_id].get('use_ipv4', True):
+                    transfers[req_id]['use_ipv4'] = True
 
         except Exception:
             logging.critical("Exception happened when trying to get transfer for request %s: %s" % (req_id, traceback.format_exc()))
@@ -1224,7 +1246,7 @@ def get_transfer_requests_and_source_replicas(total_workers=0, worker_number=0,
                                              'initial_request_id': req_id,
                                              'parent_request': parent_request,
                                              'account': InternalAccount('root'),
-                                             'schemes': __add_compatible_schemes(schemes=[destination_protocol], allowed_schemes=current_schemes),
+                                             'schemes': __add_compatible_schemes(schemes=[destination_protocol], allowed_schemes=SUPPORTED_PROTOCOLS),
                                              # 'src_urls': [source_url],
                                              'sources': [(source_rse_name, source_url, source_rse_id, 0, 0)],
                                              'dest_urls': [dest_url],
