@@ -1,4 +1,4 @@
-# Copyright 2013-2018 CERN for the benefit of the ATLAS collaboration.
+# Copyright 2018-2020 CERN for the benefit of the ATLAS collaboration.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,9 +14,13 @@
 #
 # Authors:
 # - Cedric Serfon <cedric.serfon@cern.ch>, 2018-2019
+# - Martin Barisits <martin.barisits@cern.ch>, 2018-2019
 # - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
-# - Brandon White <bjwhite@fnal.gov>, 2019-2020
+# - Brandon White <bjwhite@fnal.gov>, 2019
 # - Thomas Beermann <thomas.beermann@cern.ch>, 2020
+# - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
+# - Eli Chadwick <eli.chadwick@stfc.ac.uk>, 2020
+# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
 #
 # PY3K COMPATIBLE
 
@@ -120,6 +124,7 @@ def minos(bulk=1000, once=False, sleep_time=60):
             # Process the bad and suspicious files
             # The scope, name, rse_id are extracted and filled into the bad_replicas table
             for account, reason, state in bad_replicas:
+                vo = account.vo
                 pfns = bad_replicas[(account, reason, state)]
                 logging.info(prepend_str + 'Declaring %s replicas with state %s and reason %s' % (len(pfns), str(state), reason))
                 session = get_session()
@@ -134,7 +139,7 @@ def minos(bulk=1000, once=False, sleep_time=60):
                             schemes[scheme] = []
                         schemes[scheme].append(pfn)
                     for scheme in schemes:
-                        _, tmp_dict_rse, tmp_unknown_replicas = get_pfn_to_rse(schemes[scheme])
+                        _, tmp_dict_rse, tmp_unknown_replicas = get_pfn_to_rse(schemes[scheme], vo=vo)
                         for rse_id in tmp_dict_rse:
                             if rse_id not in dict_rse:
                                 dict_rse[rse_id] = []
@@ -146,7 +151,8 @@ def minos(bulk=1000, once=False, sleep_time=60):
                         bulk_delete_bad_pfns(pfns=unknown_replicas, session=None)
 
                     for rse_id in dict_rse:
-                        logging.debug(prepend_str + 'Running on RSE %s with %s replicas' % (get_rse_name(rse_id=rse_id), len(dict_rse[rse_id])))
+                        vo_str = '' if vo == 'def' else ' on VO ' + vo
+                        logging.debug(prepend_str + 'Running on RSE %s%s with %s replicas' % (get_rse_name(rse_id=rse_id), vo_str, len(dict_rse[rse_id])))
                         nchunk = 0
                         tot_chunk = int(math.ceil(len(dict_rse[rse_id]) / chunk_size))
                         for chunk in chunks(dict_rse[rse_id], chunk_size):
@@ -163,6 +169,7 @@ def minos(bulk=1000, once=False, sleep_time=60):
 
             # Now get the temporary unavailable and update the replicas states
             for account, reason, expires_at in temporary_unvailables:
+                vo = account.vo
                 pfns = temporary_unvailables[(account, reason, expires_at)]
                 logging.info(prepend_str + 'Declaring %s replicas temporary available with timeout %s and reason %s' % (len(pfns), str(expires_at), reason))
                 logging.debug(prepend_str + 'Extracting RSEs')
@@ -177,7 +184,7 @@ def minos(bulk=1000, once=False, sleep_time=60):
                         schemes[scheme] = []
                     schemes[scheme].append(pfn)
                 for scheme in schemes:
-                    _, tmp_dict_rse, tmp_unknown_replicas = get_pfn_to_rse(schemes[scheme])
+                    _, tmp_dict_rse, tmp_unknown_replicas = get_pfn_to_rse(schemes[scheme], vo=vo)
                     for rse_id in tmp_dict_rse:
                         if rse_id not in dict_rse:
                             dict_rse[rse_id] = []
@@ -192,8 +199,9 @@ def minos(bulk=1000, once=False, sleep_time=60):
                 for rse_id in dict_rse:
                     replicas = []
                     rse = get_rse_name(rse_id=rse_id, session=None)
-                    logging.debug(prepend_str + 'Running on RSE %s' % rse)
-                    for rep in get_did_from_pfns(pfns=dict_rse[rse_id], rse_id=None, session=None):
+                    rse_vo_str = rse if vo == 'def' else '{} on {}'.format(rse, vo)
+                    logging.debug(prepend_str + 'Running on RSE %s' % rse_vo_str)
+                    for rep in get_did_from_pfns(pfns=dict_rse[rse_id], rse_id=None, vo=vo, session=None):
                         for pfn in rep:
                             scope = rep[pfn]['scope']
                             name = rep[pfn]['name']
@@ -202,7 +210,7 @@ def minos(bulk=1000, once=False, sleep_time=60):
                     # We update the replicas states to TEMPORARY_UNAVAILABLE
                     # then insert a row in the bad_replicas table. TODO Update the row if it already exists
                     # then delete the corresponding rows into the bad_pfns table
-                    logging.debug(prepend_str + 'Running on %s replicas on RSE %s' % (len(replicas), rse))
+                    logging.debug(prepend_str + 'Running on %s replicas on RSE %s' % (len(replicas), rse_vo_str))
                     nchunk = 0
                     tot_chunk = int(math.ceil(len(replicas) / float(chunk_size)))
                     session = get_session()
@@ -233,7 +241,7 @@ def minos(bulk=1000, once=False, sleep_time=60):
                                     elif expires_at < datetime.now():
                                         logging.info('%s PFN %s expiration time (%s) is older than now and is not in unavailable state. Removing the PFNs from bad_pfns', prepend_str, str(rep['pfn']), expires_at)
                                         bulk_delete_bad_pfns(pfns=[rep['pfn']], session=None)
-                                except (DataIdentifierNotFound, ReplicaNotFound) as error:
+                                except (DataIdentifierNotFound, ReplicaNotFound):
                                     logging.error(prepend_str + 'Will remove %s from the list of bad PFNs' % str(rep['pfn']))
                                     bulk_delete_bad_pfns(pfns=[rep['pfn']], session=None)
                             session = get_session()
