@@ -25,8 +25,6 @@
 # - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
 # - Martin Barisits <martin.barisits@cern.ch>, 2020
 # - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
-#
-# PY3K COMPATIBLE
 
 from __future__ import print_function
 
@@ -49,10 +47,11 @@ from rucio.api.authentication import (get_auth_oidc, get_auth_token_gss,
                                       validate_auth_token)
 from rucio.common.config import config_get
 from rucio.common.exception import AccessDenied, IdentityError, RucioException
-from rucio.common.utils import date_to_str, generate_http_error, urlparse
+from rucio.common.utils import date_to_str, urlparse
 from rucio.web.rest.common import RucioController, check_accept_header_wrapper
-
 # Extra modules: Only imported if available
+from rucio.web.rest.utils import generate_http_error
+
 EXTRA_MODULES = {'onelogin': False}
 
 for extra_module in EXTRA_MODULES:
@@ -65,7 +64,7 @@ for extra_module in EXTRA_MODULES:
 if EXTRA_MODULES['onelogin']:
     from onelogin.saml2.auth import OneLogin_Saml2_Auth
     from web import cookies
-    from rucio.web.ui.common.utils import prepare_webpy_request
+    from rucio.web.ui.common.utils import prepare_saml_request
 
 URLS = (
     '/userpass', 'UserPass',
@@ -963,7 +962,7 @@ class SAML(RucioController):
 
         request = ctx.env
         data = dict(param_input())
-        req = prepare_webpy_request(request, data)
+        req = prepare_saml_request(request, data)
         auth = OneLogin_Saml2_Auth(req, custom_base_path=SAML_PATH)
 
         header('X-Rucio-SAML-Auth-URL', auth.login())
@@ -976,9 +975,7 @@ class SAML(RucioController):
             return "SAML not configured on the server side."
 
         SAML_PATH = config_get('saml', 'config_path')
-        request = ctx.env
-        data = dict(param_input())
-        req = prepare_webpy_request(request, data)
+        req = prepare_saml_request(ctx.env, dict(param_input()))
         auth = OneLogin_Saml2_Auth(req, custom_base_path=SAML_PATH)
 
         auth.process_response()
