@@ -39,7 +39,7 @@ from six import string_types
 from rucio.api.replica import (add_replicas, list_replicas, list_dataset_replicas,
                                list_dataset_replicas_bulk, delete_replicas,
                                get_did_from_pfns, update_replicas_states,
-                               declare_bad_file_replicas, add_bad_pfns, get_suspicious_files,
+                               declare_bad_file_replicas, add_bad_dids, add_bad_pfns, get_suspicious_files,
                                declare_suspicious_file_replicas, list_bad_replicas_status,
                                get_bad_replicas_summary, list_datasets_per_rse,
                                set_tombstone, list_dataset_replicas_vp)
@@ -52,8 +52,9 @@ from rucio.common.exception import (AccessDenied, DataIdentifierAlreadyExists, I
                                     InvalidObject, ScopeNotFound)
 from rucio.common.replica_sorter import sort_random, sort_geoip, sort_closeness, sort_dynamic, sort_ranking
 from rucio.common.utils import parse_response, APIEncoder, render_json_list
-from rucio.db.sqla.constants import BadFilesStatus
-from rucio.web.rest.flaskapi.v1.common import check_accept_header_wrapper_flask, try_stream, parse_scope_name, request_auth_env, response_headers, request_header_ensure_string
+from rucio.db.sqla.constants import BadFilesStatus, ReplicaState
+from rucio.web.rest.flaskapi.v1.common import (check_accept_header_wrapper_flask, try_stream, parse_scope_name,
+                                               request_auth_env, response_headers, request_header_ensure_string)
 from rucio.web.rest.utils import generate_http_error_flask
 
 try:
@@ -943,6 +944,62 @@ class ReplicasRSE(MethodView):
             return str(error), 500
 
 
+class BadDIDs(MethodView):
+
+    def post(self):
+        """
+        Declare a list of bad replicas by DID.
+
+        .. :quickref: BadDIDs; Declare bad replicas by DID.
+
+        :<json string pfns: The list of PFNs.
+        :<json string reason: The reason of the loss.
+        :<json string state: The state is eiher BAD, SUSPICIOUS or TEMPORARY_UNAVAILABLE.
+        :<json string expires_at: The expiration date. Only apply to TEMPORARY_UNAVAILABLE.
+        :resheader Content-Type: application/x-json-string
+        :status 201: Created.
+        :status 400: Cannot decode json parameter list.
+        :status 401: Invalid auth token.
+        :status 404: Replica not found.
+        :status 500: Internal Error.
+        :returns: A list of not successfully declared files.
+        """
+
+        json_data = request.data
+        dids = []
+        rse = None
+        reason = None
+        state = None
+        expires_at = None
+        try:
+            params = parse_response(json_data)
+            if 'dids' in params:
+                dids = params['dids']
+            if 'rse' in params:
+                rse = params['rse']
+            if 'reason' in params:
+                reason = params['reason']
+            state = ReplicaState.BAD
+            if 'expires_at' in params and params['expires_at']:
+                expires_at = datetime.strptime(params['expires_at'], "%Y-%m-%dT%H:%M:%S.%f")
+            not_declared_files = add_bad_dids(dids=dids, rse=rse, issuer=request.environ.get('issuer'), state=state,
+                                              reason=reason, expires_at=expires_at, vo=request.environ.get('vo'))
+        except (ValueError, InvalidType) as error:
+            return generate_http_error_flask(400, 'ValueError', error.args[0])
+        except AccessDenied as error:
+            return generate_http_error_flask(401, 'AccessDenied', error.args[0])
+        except ReplicaNotFound as error:
+            return generate_http_error_flask(404, 'ReplicaNotFound', error.args[0])
+        except Duplicate as error:
+            return generate_http_error_flask(409, 'Duplicate', error.args[0])
+        except RucioException as error:
+            return generate_http_error_flask(500, error.__class__.__name__, error.args[0])
+        except Exception as error:
+            print(format_exc())
+            return str(error), 500
+        return Response(dumps(not_declared_files), status=201, content_type='application/json')
+
+
 class BadPFNs(MethodView):
 
     def post(self):
@@ -1053,6 +1110,8 @@ def blueprint(no_doc=True):
     bp.add_url_rule('/bad/summary', view_func=bad_replicas_summary_view, methods=['get', ])
     bad_replicas_pfn_view = BadPFNs.as_view('add_bad_pfns')
     bp.add_url_rule('/bad/pfns', view_func=bad_replicas_pfn_view, methods=['post', ])
+    bad_replicas_dids_view = BadDIDs.as_view('add_bad_dids')
+    bp.add_url_rule('/bad/dids', view_func=bad_replicas_dids_view, methods=['post', ])
     replicas_rse_view = ReplicasRSE.as_view('replicas_rse')
     bp.add_url_rule('/rse/<rse>', view_func=replicas_rse_view, methods=['get', ])
     bad_replicas_view = BadReplicas.as_view('bad_replicas')
@@ -1074,6 +1133,7 @@ def blueprint(no_doc=True):
         bp.add_url_rule('/bad/states/', view_func=bad_replicas_states_view, methods=['get', ])
         bp.add_url_rule('/bad/summary/', view_func=bad_replicas_summary_view, methods=['get', ])
         bp.add_url_rule('/bad/pfns/', view_func=bad_replicas_pfn_view, methods=['post', ])
+        bp.add_url_rule('/bad/dids/', view_func=bad_replicas_dids_view, methods=['post', ])
         bp.add_url_rule('/rse/<rse>/', view_func=replicas_rse_view, methods=['get', ])
         bp.add_url_rule('/bad/', view_func=bad_replicas_view, methods=['post', ])
         bp.add_url_rule('/dids/', view_func=replicas_dids_view, methods=['post', ])
