@@ -1,5 +1,6 @@
 #!/usr/bin/env python
-# Copyright 2012-2018 CERN for the benefit of the ATLAS collaboration.
+# -*- coding: utf-8 -*-
+# Copyright 2014-2020 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,27 +18,31 @@
 # - Vincent Garonne <vincent.garonne@cern.ch>, 2014-2017
 # - Cedric Serfon <cedric.serfon@cern.ch>, 2014-2019
 # - Mario Lassnig <mario.lassnig@cern.ch>, 2014-2018
+# - Thomas Beermann <thomas.beermann@cern.ch>, 2018
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
-#
-# PY3K COMPATIBLE
+# - James Perry <j.perry@epcc.ed.ac.uk>, 2019-2020
+# - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
+# - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
+# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
 
 from __future__ import print_function
+
+from logging import getLogger, StreamHandler, DEBUG
 from traceback import format_exc
+
+from web import application, ctx, header, seeother, InternalError
+
+from rucio.api.replica import list_replicas
+from rucio.common.exception import RucioException, DataIdentifierNotFound, ReplicaNotFound
+from rucio.common.replica_sorter import sort_random, sort_geoip, sort_closeness, sort_ranking, sort_dynamic, site_selector
+from rucio.common.schema import get_schema_value
+from rucio.web.rest.common import RucioController, check_accept_header_wrapper
+from rucio.web.rest.utils import generate_http_error
+
 try:
     from urlparse import parse_qs
 except ImportError:
     from urllib.parse import parse_qs
-from web import application, ctx, header, seeother, InternalError
-
-from logging import getLogger, StreamHandler, DEBUG
-
-from rucio.api.replica import list_replicas
-from rucio.common.objectstore import connect, get_signed_urls
-from rucio.common.exception import RucioException, DataIdentifierNotFound, ReplicaNotFound
-from rucio.common.replica_sorter import sort_random, sort_geoip, sort_closeness, sort_ranking, sort_dynamic, site_selector
-from rucio.common.schema import SCOPE_NAME_REGEXP
-from rucio.common.utils import generate_http_error
-from rucio.web.rest.common import RucioController, check_accept_header_wrapper
 
 
 LOGGER = getLogger("rucio.rucio")
@@ -45,8 +50,8 @@ SH = StreamHandler()
 SH.setLevel(DEBUG)
 LOGGER.addHandler(SH)
 
-URLS = ('%s/metalink?$' % SCOPE_NAME_REGEXP, 'MetaLinkRedirector',
-        '%s/?$' % SCOPE_NAME_REGEXP, 'HeaderRedirector')
+URLS = ('%s/metalink?$' % get_schema_value('SCOPE_NAME_REGEXP'), 'MetaLinkRedirector',
+        '%s/?$' % get_schema_value('SCOPE_NAME_REGEXP'), 'HeaderRedirector')
 
 
 class MetaLinkRedirector(RucioController):
@@ -74,7 +79,7 @@ class MetaLinkRedirector(RucioController):
         header('Access-Control-Allow-Methods', '*')
         header('Access-Control-Allow-Credentials', 'true')
 
-        dids, schemes, select = [{'scope': scope, 'name': name}], ['http', 'https', 's3+rucio', 's3+https', 'root', 'gsiftp', 'srm', 'davs'], None
+        dids, schemes, select = [{'scope': scope, 'name': name}], ['http', 'https', 'root', 'gsiftp', 'srm', 'davs'], None
 
         # set the correct client IP
         client_ip = ctx.env.get('HTTP_X_FORWARDED_FOR')
@@ -101,8 +106,11 @@ class MetaLinkRedirector(RucioController):
             if 'site' in params:
                 client_location['site'] = params['site'][0]
 
+        # get vo if given
+        vo = ctx.env.get('HTTP_X_RUCIO_VO', 'def')
+
         try:
-            tmp_replicas = [rep for rep in list_replicas(dids=dids, schemes=schemes, client_location=client_location)]
+            tmp_replicas = [rep for rep in list_replicas(dids=dids, schemes=schemes, client_location=client_location, vo=vo)]
 
             if not tmp_replicas:
                 raise ReplicaNotFound('no redirection possible - cannot find the DID')
@@ -194,7 +202,7 @@ class HeaderRedirector(RucioController):
         try:
 
             # use the default HTTP protocols if no scheme is given
-            select, rse, site, schemes = 'random', None, None, ['davs', 'http', 'https', 's3+rucio']
+            select, rse, site, schemes = 'random', None, None, ['davs', 'http', 'https']
 
             client_ip = ctx.env.get('HTTP_X_FORWARDED_FOR')
             if client_ip is None:
@@ -234,9 +242,13 @@ class HeaderRedirector(RucioController):
                 header('Link', '<%s/metalink?schemes=%s&select=%s>; rel=describedby; type="application/metalink+xml"' % (cleaned_url, schemes, select))
                 schemes = [schemes]  # list_replicas needs a list
 
-            replicas = [r for r in list_replicas(dids=[{'scope': scope, 'name': name, 'type': 'FILE'}], schemes=schemes, client_location=client_location)]
+            # get vo if given
+            vo = ctx.env.get('HTTP_X_RUCIO_VO', 'def')
 
-            selected_url, selected_rse = None, None
+            replicas = [r for r in list_replicas(dids=[{'scope': scope, 'name': name, 'type': 'FILE'}],
+                                                 schemes=schemes, client_location=client_location, vo=vo)]
+
+            selected_url = None
             for r in replicas:
                 if r['rses']:
                     dictreplica = {}
@@ -244,7 +256,6 @@ class HeaderRedirector(RucioController):
                     if rse:
                         if rse in r['rses'] and r['rses'][rse]:
                             selected_url = r['rses'][rse][0]
-                            selected_rse = rse
                         else:
                             raise ReplicaNotFound('no redirection possible - no valid RSE for HTTP redirection found')
                     else:
@@ -260,7 +271,7 @@ class HeaderRedirector(RucioController):
                             raise ReplicaNotFound('no redirection possible - no valid RSE for HTTP redirection found')
 
                         elif site:
-                            rep = site_selector(dictreplica, site)
+                            rep = site_selector(dictreplica, site, vo)
                             if rep:
                                 selected_url = rep[0]
                             else:
@@ -279,19 +290,7 @@ class HeaderRedirector(RucioController):
 
                             selected_url = rep[0]
 
-                        for rep in r['rses']:
-                            for replica in r['rses'][rep]:
-                                if selected_url == replica:
-                                    selected_rse = rep
-
             if selected_url:
-                if selected_url.startswith('s3+rucio://'):
-                    connect(selected_rse, selected_url)
-                    signed_URLS = get_signed_urls([selected_url],
-                                                  rse=selected_rse,
-                                                  operation='read')
-                    raise seeother(signed_URLS[selected_url])
-
                 raise seeother(selected_url)
 
             raise ReplicaNotFound('no redirection possible - file does not exist')
